@@ -2,17 +2,21 @@
 
 set -xeuo pipefail
 
-# N.B.: The 1st listed version will supply the default `python` on the PATH; otherwise order does
-# not matter.
+DEFAULT_PYTHON_VERSION=3.14
+
+# N.B.: 3.12 is the default system python for ubuntu 24.04 and software-properties-common uses it.
+# There may be a way to substitute the deadsnakes version but I have not found it; so we uninstall
+# via `apt autoremove` and let pyenv install a 3.12.
+
 # See: https://launchpad.net/~deadsnakes/+archive/ubuntu/ppa
 DEADSNAKES_VERSIONS=(
-  "3.11 {dev,venv,distutils}=3.11.13"
   "3.7 {dev,venv,distutils}=3.7.17"
   "3.8 {dev,venv,distutils}=3.8.20"
-  "3.9 {dev,venv,distutils}=3.9.23"
-  "3.10 {dev,venv,distutils}=3.10.18"
-  "3.13 {dev,venv}=3.13.7"
-  "3.14 {dev,venv}=3.14.0~rc3"
+  "3.9 {dev,venv,distutils}=3.9.24"
+  "3.10 {dev,venv,distutils}=3.10.19"
+  "3.11 {dev,venv,distutils}=3.11.14"
+  "3.13 {dev,venv}=3.13.8"
+  "3.14 {dev,venv}=3.14.0"
 )
 
 add-apt-repository --yes --ppa deadsnakes
@@ -20,11 +24,6 @@ for entry in "${DEADSNAKES_VERSIONS[@]}"; do
   version="${entry/ */}"
   packages="${entry/* /}*"
   DEBIAN_FRONTEND=noninteractive apt install --yes $(eval echo python${version}-${packages})
-
-  # Let the 1st version supply the default `python`.
-  if [[ ! -e "/usr/bin/python" ]]; then
-    ln -s "$(which "python${version}")" "/usr/bin/python"
-  fi
 done
 add-apt-repository --yes --remove --ppa deadsnakes
 DEBIAN_FRONTEND=noninteractive apt remove --yes software-properties-common
@@ -36,7 +35,7 @@ PYENV_VERSIONS=(
   2.7.18
   3.5.10
   3.6.15
-  3.12.11
+  3.12.12
   pypy2.7-7.3.20
   pypy3.5-7.0.0
   pypy3.6-7.3.3
@@ -64,3 +63,5 @@ for version in "${PYENV_VERSIONS[@]}"; do
 
   ln -s "${exe_path}" "/usr/bin/${exe}"
 done
+
+ln -s "$(which "python${DEFAULT_PYTHON_VERSION}")" "/usr/bin/python"
\ No newline at end of file
