@@ -1,11 +1,11 @@
-# Copyright 2022 Pants project contributors (see CONTRIBUTORS.md).
+# Copyright 2022 Pex project contributors.
 # Licensed under the Apache License, Version 2.0 (see LICENSE).
 
 from __future__ import absolute_import
 
+from pex.dist_metadata import Distribution
 from pex.pep_503 import ProjectName
-from pex.third_party.pkg_resources import Distribution
-from pex.typing import TYPE_CHECKING, cast
+from pex.typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
     import attr  # vendor:skip
@@ -21,9 +21,9 @@ class FingerprintedDistribution(object):
     @property
     def location(self):
         # type: () -> str
-        return cast(str, self.distribution.location)
+        return self.distribution.location
 
     @property
     def project_name(self):
         # type: () -> ProjectName
-        return ProjectName(self.distribution)
+        return self.distribution.metadata.project_name
