@@ -1,43 +1,137 @@
-# Copyright 2014 Pants project contributors (see CONTRIBUTORS.md).
+# Copyright 2014 Pex project contributors.
 # Licensed under the Apache License, Version 2.0 (see LICENSE).
 
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import
 
 import ast
+import itertools
 import os
-import re
 import sys
-import warnings
-import zipfile
-from distutils import sysconfig
 from site import USER_SITE
 from types import ModuleType
 
-from pex import third_party
+from pex import bootstrap, repl
 from pex.bootstrap import Bootstrap
 from pex.common import die
-from pex.compatibility import PY3
+from pex.dist_metadata import CallableEntryPoint, Distribution, ModuleEntryPoint, parse_entry_point
 from pex.environment import PEXEnvironment
 from pex.executor import Executor
 from pex.finders import get_entry_point_from_console_script, get_script_from_distributions
+from pex.fingerprinted_distribution import FingerprintedDistribution
+from pex.globals import Globals
 from pex.inherit_path import InheritPath
-from pex.interpreter import PythonInterpreter
+from pex.interpreter import PythonIdentity, PythonInterpreter
+from pex.layout import Layout
 from pex.orderedset import OrderedSet
+from pex.os import safe_execv
 from pex.pex_info import PexInfo
+from pex.subprocess import subprocess_daemon_kwargs
 from pex.targets import LocalInterpreter
-from pex.third_party.pkg_resources import Distribution, EntryPoint, find_distributions
 from pex.tracer import TRACER
 from pex.typing import TYPE_CHECKING
-from pex.util import iter_pth_paths, named_temporary_file
+from pex.util import named_temporary_file
 from pex.variables import ENV, Variables
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, Iterable, Iterator, List, Mapping, Optional, Set, Tuple, TypeVar
+    from typing import (
+        Any,
+        Dict,
+        Iterable,
+        Iterator,
+        List,
+        Mapping,
+        NoReturn,
+        Optional,
+        Sequence,
+        Tuple,
+        TypeVar,
+        Union,
+    )
 
     _K = TypeVar("_K")
     _V = TypeVar("_V")
 
 
+class IsolatedSysPath(object):
+    @staticmethod
+    def _expand_paths(*paths):
+        # type: (*str) -> OrderedSet[str]
+        def iter_synonyms(path):
+            yield path
+            yield os.path.abspath(path)
+            yield os.path.realpath(path)
+
+        return OrderedSet(itertools.chain.from_iterable(iter_synonyms(path) for path in paths))
+
+    @classmethod
+    def for_pex(
+        cls,
+        interpreter,  # type: Union[PythonInterpreter, PythonIdentity]
+        pex,  # type: str
+        pex_pex=None,  # type: Optional[str]
+    ):
+        # type: (...) -> IsolatedSysPath
+        ident = interpreter.identity if isinstance(interpreter, PythonInterpreter) else interpreter
+        sys_path = OrderedSet(ident.sys_path)
+        sys_path.add(pex)
+        sys_path.add(Bootstrap.locate().path)
+        if pex_pex:
+            sys_path.add(pex_pex)
+
+        site_packages = OrderedSet()  # type: OrderedSet[str]
+        for site_lib in ident.site_packages:
+            TRACER.log("Discarding site packages path: {site_lib}".format(site_lib=site_lib))
+            site_packages.add(site_lib.path)
+
+        extras_paths = OrderedSet()  # type: OrderedSet[str]
+        for extras_path in ident.extras_paths:
+            TRACER.log("Discarding site extras path: {extras_path}".format(extras_path=extras_path))
+            extras_paths.add(extras_path)
+
+        return cls(
+            sys_path=sys_path,
+            site_packages=site_packages,
+            extras_paths=extras_paths,
+            is_venv=ident.is_venv,
+        )
+
+    def __init__(
+        self,
+        sys_path,  # type: Iterable[str]
+        site_packages,  # type: Iterable[str]
+        extras_paths=(),  # type: Iterable[str]
+        is_venv=False,  # type: bool
+    ):
+        # type: (...) -> None
+        self._sys_path_entries = tuple(self._expand_paths(*sys_path))
+        self._site_packages_entries = tuple(self._expand_paths(*site_packages))
+        self._extras_paths_entries = tuple(self._expand_paths(*extras_paths))
+        self._is_venv = is_venv
+
+    @property
+    def is_venv(self):
+        # type: () -> bool
+        return self._is_venv
+
+    def __contains__(self, entry):
+        # type: (str) -> bool
+        for path in self._expand_paths(entry):
+            # N.B.: Since site packages is contained in sys.path, process rejections 1st to ensure
+            # they never sneak by.
+            if path.startswith(self._site_packages_entries):
+                return False
+            if path.startswith(self._extras_paths_entries):
+                return False
+            if path.startswith(self._sys_path_entries):
+                return True
+
+        # A sys.path entry injected by calling code. This can happen when a PEX is used as a
+        # sys.path entry via the __pex__ importer mechanism or via the old pex_bootstrapper
+        # bootstrap_pex_env API used by at least lambdex and pyuwsgi_pex. Pointedly, the AWS lambda
+        # does this, adding /var/runtime to sys.path before executing user code.
+        return False
+
+
 class PEX(object):  # noqa: T000
     """PEX, n.
 
@@ -74,16 +168,25 @@ class PEX(object):  # noqa: T000
         self._pex_info = PexInfo.from_pex(self._pex)
         self._pex_info_overrides = PexInfo.from_env(env=env)
         self._vars = env
-        self._envs = None  # type: Optional[Iterable[PEXEnvironment]]
-        self._activated_dists = None  # type: Optional[Iterable[Distribution]]
+        self._envs = None  # type: Optional[Sequence[PEXEnvironment]]
+        self._activated_dists = None  # type: Optional[Sequence[Distribution]]
+        self._layout = None  # type: Optional[Layout.Value]
         if verify_entry_point:
             self._do_entry_point_verification()
 
-    def pex_info(self):
-        # type: () -> PexInfo
+    @property
+    def layout(self):
+        # type: () -> Layout.Value
+        if self._layout is None:
+            self._layout = Layout.identify_original(self._pex)
+        return self._layout
+
+    def pex_info(self, include_env_overrides=True):
+        # type: (bool) -> PexInfo
         pex_info = self._pex_info.copy()
-        pex_info.update(self._pex_info_overrides)
-        pex_info.merge_pex_path(self._vars.PEX_PATH)
+        if include_env_overrides:
+            pex_info.update(self._pex_info_overrides)
+            pex_info.merge_pex_path(self._vars.PEX_PATH)
         return pex_info
 
     @property
@@ -93,7 +196,7 @@ class PEX(object):  # noqa: T000
 
     @property
     def _loaded_envs(self):
-        # type: () -> Iterable[PEXEnvironment]
+        # type: () -> Sequence[PEXEnvironment]
         if self._envs is None:
             # set up the local .pex environment
             pex_info = self.pex_info()
@@ -103,23 +206,42 @@ class PEX(object):  # noqa: T000
             # merged from pex_path in `PEX-INFO` and `PEX_PATH` set in the environment.
             # `PEX_PATH` entries written into `PEX-INFO` take precedence over those set
             # in the environment.
-            if pex_info.pex_path:
-                # set up other environments as specified in pex_path
-                for pex_path in filter(None, pex_info.pex_path.split(os.pathsep)):
-                    pex_info = PexInfo.from_pex(pex_path)
-                    pex_info.update(self._pex_info_overrides)
-                    envs.append(PEXEnvironment.mount(pex_path, pex_info, target=target))
-            self._envs = tuple(envs)
+            for pex_path in pex_info.pex_path:
+                # Set up other environments as specified in pex_path.
+                pex_info = PexInfo.from_pex(pex_path)
+                pex_info.update(self._pex_info_overrides)
+                envs.append(PEXEnvironment.mount(pex_path, pex_info, target=target))
+            self._envs = envs
         return self._envs
 
     def resolve(self):
         # type: () -> Iterator[Distribution]
+        """Resolves all distributions loadable from this PEX by the current interpreter."""
+        seen = set()
         for env in self._loaded_envs:
             for dist in env.resolve():
+                # N.B.: Since there can be more than one PEX env on the PEX_PATH we take care to
+                # de-dup distributions they have in common.
+                if dist in seen:
+                    continue
+                seen.add(dist)
+                yield dist
+
+    def iter_distributions(self, result_type_wheel_file=False):
+        # type: (bool) -> Iterator[FingerprintedDistribution]
+        """Iterates all distributions loadable from this PEX."""
+        seen = set()
+        for env in self._loaded_envs:
+            for dist in env.iter_distributions(result_type_wheel_file=result_type_wheel_file):
+                # N.B.: Since there can be more than one PEX env on the PEX_PATH we take care to
+                # de-dup distributions they have in common.
+                if dist in seen:
+                    continue
+                seen.add(dist)
                 yield dist
 
     def _activate(self):
-        # type: () -> Iterable[Distribution]
+        # type: () -> Sequence[Distribution]
 
         activated_dists = []  # type: List[Distribution]
         for env in self._loaded_envs:
@@ -131,7 +253,7 @@ class PEX(object):  # noqa: T000
         return activated_dists
 
     def activate(self):
-        # type: () -> Iterable[Distribution]
+        # type: () -> Sequence[Distribution]
         if self._activated_dists is None:
             # 1. Scrub the sys.path to present a minimal Python environment.
             self.patch_sys()
@@ -140,101 +262,48 @@ class PEX(object):  # noqa: T000
         return self._activated_dists
 
     @classmethod
-    def _extras_paths(cls):
-        # type: () -> Iterator[str]
-        standard_lib = sysconfig.get_python_lib(standard_lib=True)
-
-        try:
-            makefile = sysconfig.parse_makefile(  # type: ignore[attr-defined]
-                sysconfig.get_makefile_filename()
-            )
-        except (AttributeError, IOError):
-            # This is not available by default in PyPy's distutils.sysconfig or it simply is
-            # no longer available on the system (IOError ENOENT)
-            makefile = {}
-
-        extras_paths = filter(
-            None, makefile.get("EXTRASPATH", "").split(":")
-        )  # type: Iterable[str]
-        for path in extras_paths:
-            yield os.path.join(standard_lib, path)
-
-        # Handle .pth injected paths as extras.
-        sitedirs = cls._get_site_packages()
-        for pth_path in cls._scan_pth_files(sitedirs):
-            TRACER.log("Found .pth file: %s" % pth_path, V=3)
-            for extras_path in iter_pth_paths(pth_path):
-                yield extras_path
-
-    @staticmethod
-    def _scan_pth_files(dir_paths):
-        """Given an iterable of directory paths, yield paths to all .pth files within."""
-        for dir_path in dir_paths:
-            if not os.path.exists(dir_path):
-                continue
-
-            pth_filenames = (f for f in os.listdir(dir_path) if f.endswith(".pth"))
-            for pth_filename in pth_filenames:
-                yield os.path.join(dir_path, pth_filename)
-
-    @staticmethod
-    def _get_site_packages():
-        # type: () -> Set[str]
-        try:
-            from site import getsitepackages
-
-            return set(getsitepackages())
-        except ImportError:
-            return set()
-
-    @classmethod
-    def site_libs(cls):
-        # type: () -> Set[str]
-        site_libs = cls._get_site_packages()
-        site_libs.update(
-            [
-                sysconfig.get_python_lib(plat_specific=False),
-                sysconfig.get_python_lib(plat_specific=True),
-            ]
-        )
-        # On windows getsitepackages() returns the python stdlib too.
-        if sys.prefix in site_libs:
-            site_libs.remove(sys.prefix)
-        real_site_libs = set(os.path.realpath(path) for path in site_libs)
-        return site_libs | real_site_libs
-
-    @classmethod
-    def _tainted_path(cls, path, site_libs):
-        # type: (str, Iterable[str]) -> bool
-        paths = frozenset([path, os.path.realpath(path)])
-        return any(path.startswith(site_lib) for site_lib in site_libs for path in paths)
-
-    @classmethod
-    def minimum_sys_modules(cls, site_libs, modules=None):
-        # type: (Iterable[str], Optional[Mapping[str, ModuleType]]) -> Mapping[str, ModuleType]
+    def minimum_sys_modules(
+        cls,
+        isolated_sys_path,  # type: IsolatedSysPath
+        modules=None,  # type: Optional[Mapping[str, ModuleType]]
+    ):
+        # type: (...) -> Mapping[str, ModuleType]
         """Given a set of site-packages paths, return a "clean" sys.modules.
 
-        When importing site, modules within sys.modules have their __path__'s populated with
-        additional paths as defined by *-nspkg.pth in site-packages, or alternately by distribution
-        metadata such as *.dist-info/namespace_packages.txt.  This can possibly cause namespace
-        packages to leak into imports despite being scrubbed from sys.path.
+        When importing site, modules within `sys.modules` have their `__path__`s populated with
+        additional paths as defined by `*-nspkg.pth` in `site-packages`, or alternately by
+        distribution metadata such as `*.dist-info/namespace_packages.txt`. This can possibly cause
+        namespace packages to leak into imports despite being scrubbed from `sys.path`.
 
-        NOTE: This method mutates modules' __path__ attributes in sys.modules, so this is currently an
-        irreversible operation.
+        NOTE: This method mutates modules' `__path__` attributes in `sys.modules`, so this is
+        currently an irreversible operation.
         """
 
+        is_venv = isolated_sys_path.is_venv
         modules = modules or sys.modules
         new_modules = {}
 
         for module_name, module in modules.items():
             # Tainted modules should be dropped.
             module_file = getattr(module, "__file__", None)
-            if module_file and cls._tainted_path(module_file, site_libs):
+            if (
+                # The `_virtualenv` module is a known special case provided by the virtualenv
+                # project. It should not be un-imported or else the virtualenv importer it installs
+                # for performing needed patches to the `distutils` stdlib breaks.
+                #
+                # See:
+                # + https://github.com/pex-tool/pex/issues/992
+                # + https://github.com/pypa/virtualenv/pull/1688
+                (not is_venv or module_name != "_virtualenv")
+                and module_file
+                and module_file not in isolated_sys_path
+            ):
                 TRACER.log("Dropping %s" % (module_name,), V=3)
                 continue
 
             module_path = getattr(module, "__path__", None)
-            # Untainted non-packages (builtin modules) need no further special handling and can stay.
+            # Untainted non-packages (builtin modules) need no further special handling and can
+            # stay.
             if module_path is None:
                 new_modules[module_name] = module
                 continue
@@ -246,7 +315,7 @@ class PEX(object):  # noqa: T000
 
             # Drop tainted package paths.
             for k in reversed(range(len(module_path))):
-                if cls._tainted_path(module_path[k], site_libs):
+                if module_path[k] not in isolated_sys_path:
                     TRACER.log("Scrubbing %s.__path__: %s" % (module_name, module_path[k]), V=3)
                     module_path.pop(k)
 
@@ -276,34 +345,37 @@ class PEX(object):  # noqa: T000
         return pythonpath
 
     @classmethod
-    def minimum_sys_path(cls, site_libs, inherit_path):
-        # type: (Iterable[str], InheritPath.Value) -> Tuple[List[str], Mapping[str, Any]]
+    def minimum_sys_path(
+        cls,
+        isolated_sys_path,  # type: IsolatedSysPath
+        inherit_path,  # type: InheritPath.Value
+    ):
+        # type: (...) -> Tuple[List[str], Mapping[str, Any]]
         scrub_paths = OrderedSet()  # type: OrderedSet[str]
-        site_distributions = OrderedSet()  # type: OrderedSet[str]
-        user_site_distributions = OrderedSet()  # type: OrderedSet[str]
+        site_paths = OrderedSet()  # type: OrderedSet[str]
+        user_site_paths = OrderedSet()  # type: OrderedSet[str]
 
-        def all_distribution_paths(path):
+        def all_paths(path):
             # type: (Optional[str]) -> Iterable[str]
             if path is None:
                 return ()
-            locations = set(dist.location for dist in find_distributions(path))
-            return {path} | locations | set(os.path.realpath(path) for path in locations)
+            return path, os.path.realpath(path)
 
         for path_element in sys.path:
-            if cls._tainted_path(path_element, site_libs):
+            if path_element not in isolated_sys_path:
                 TRACER.log("Tainted path element: %s" % path_element)
-                site_distributions.update(all_distribution_paths(path_element))
+                site_paths.update(all_paths(path_element))
             else:
                 TRACER.log("Not a tainted path element: %s" % path_element, V=2)
 
-        user_site_distributions.update(all_distribution_paths(USER_SITE))
+        user_site_paths.update(all_paths(USER_SITE))
 
         if inherit_path == InheritPath.FALSE:
-            scrub_paths = OrderedSet(site_distributions)
-            scrub_paths.update(user_site_distributions)
-            for path in user_site_distributions:
+            scrub_paths = OrderedSet(site_paths)
+            scrub_paths.update(user_site_paths)
+            for path in user_site_paths:
                 TRACER.log("Scrubbing from user site: %s" % path)
-            for path in site_distributions:
+            for path in site_paths:
                 TRACER.log("Scrubbing from site-packages: %s" % path)
 
         scrubbed_sys_path = list(OrderedSet(sys.path) - scrub_paths)
@@ -346,24 +418,18 @@ class PEX(object):  # noqa: T000
 
         return scrubbed_sys_path, scrubbed_importer_cache
 
-    @classmethod
-    def minimum_sys(cls, inherit_path):
+    def minimum_sys(self, inherit_path):
         # type: (InheritPath.Value) -> Tuple[List[str], Mapping[str, Any], Mapping[str, ModuleType]]
         """Return the minimum sys necessary to run this interpreter, a la python -S.
 
         :returns: (sys.path, sys.path_importer_cache, sys.modules) tuple of a
           bare python installation.
         """
-        site_libs = cls.site_libs()
-        for site_lib in site_libs:
-            TRACER.log("Found site-library: %s" % site_lib)
-        for extras_path in cls._extras_paths():
-            TRACER.log("Found site extra: %s" % extras_path)
-            site_libs.add(extras_path)
-        site_libs = set(os.path.normpath(path) for path in site_libs)
-
-        sys_path, sys_path_importer_cache = cls.minimum_sys_path(site_libs, inherit_path)
-        sys_modules = cls.minimum_sys_modules(site_libs)
+        isolated_sys_path = IsolatedSysPath.for_pex(
+            interpreter=self._interpreter, pex=self._pex, pex_pex=self._vars.PEX
+        )
+        sys_path, sys_path_importer_cache = self.minimum_sys_path(isolated_sys_path, inherit_path)
+        sys_modules = self.minimum_sys_modules(isolated_sys_path)
 
         return sys_path, sys_path_importer_cache, sys_modules
 
@@ -387,12 +453,31 @@ class PEX(object):  # noqa: T000
             patch_dict(sys.path_importer_cache, path_importer_cache)
             patch_dict(sys.modules, modules)
 
+            # N.B.: These hooks may contain custom code set via site.py or `.pth` files that we've
+            # just scrubbed. Reset them to factory defaults to avoid scrubbed code.
+            sys.displayhook = sys.__displayhook__
+            sys.excepthook = sys.__excepthook__  # type: ignore[assignment] # Builtin typeshed bug.
+
         new_sys_path, new_sys_path_importer_cache, new_sys_modules = self.minimum_sys(inherit_path)
 
         if self._vars.PEX_EXTRA_SYS_PATH:
-            TRACER.log("Adding %s to sys.path" % self._vars.PEX_EXTRA_SYS_PATH)
-            new_sys_path.extend(self._vars.PEX_EXTRA_SYS_PATH.split(":"))
-        TRACER.log("New sys.path: %s" % new_sys_path)
+            TRACER.log(
+                "Adding {} to sys.path".format(os.pathsep.join(self._vars.PEX_EXTRA_SYS_PATH))
+            )
+            extra_sys_path = self._vars.PEX_EXTRA_SYS_PATH
+            new_sys_path.extend(extra_sys_path)
+
+            # Let Python subprocesses see the same sys.path additions we see. If those Python
+            # subprocesses are PEX subprocesses, they'll do their own (re-)scrubbing as needed.
+            if inherit_path is InheritPath.FALSE:
+                pythonpath_entries = extra_sys_path
+            else:
+                raw_pythonpath = os.environ.get(self._PYTHONPATH)
+                pythonpath = tuple(raw_pythonpath.split(os.pathsep)) if raw_pythonpath else ()
+                pythonpath_entries = pythonpath + extra_sys_path
+            os.environ[self._PYTHONPATH] = os.pathsep.join(pythonpath_entries)
+
+        TRACER.log("New sys.path: {}".format(new_sys_path))
 
         patch_all(new_sys_path, new_sys_path_importer_cache, new_sys_modules)
 
@@ -454,77 +539,48 @@ class PEX(object):  # noqa: T000
         return self._pex
 
     def execute(self):
-        # type: () -> None
+        # type: () -> Any
         """Execute the PEX.
 
         This function makes assumptions that it is the last function called by the interpreter.
         """
-        teardown_verbosity = self._vars.PEX_TEARDOWN_VERBOSE
-        try:
-            if self._vars.PEX_TOOLS:
-                if not self._pex_info.includes_tools:
-                    die(
-                        "The PEX_TOOLS environment variable was set, but this PEX was not built "
-                        "with tools (Re-build the PEX file with `pex --include-tools ...`)"
+        pex_file = self._vars.PEX
+        if self._vars.PEX_TOOLS:
+            if not self._pex_info.includes_tools:
+                die(
+                    "The PEX_TOOLS environment variable was set, but this PEX was not built "
+                    "with tools (Re-build the PEX file with `pex --include-tools ...`)"
+                )
+
+            from pex.tools import main as tools
+
+            sys.exit(tools.main(pex=PEX(pex_file or sys.argv[0])))
+
+        self.activate()
+
+        if pex_file:
+            try:
+                from setproctitle import setproctitle  # type: ignore[import]
+
+                setproctitle(
+                    "{python} {pex_file} {args}".format(
+                        python=sys.executable,
+                        pex_file=pex_file,
+                        args=" ".join(sys.argv[1:]),
                     )
+                )
+            except ImportError:
+                TRACER.log(
+                    "Not setting process title since setproctitle is not available in "
+                    "{pex_file}".format(pex_file=pex_file),
+                    V=3,
+                )
 
-                from pex.tools import main as tools
-
-                exit_value = tools.main(pex=PEX(sys.argv[0]))
-            else:
-                self.activate()
-
-                pex_file = os.environ.get("PEX", None)
-                if pex_file:
-                    try:
-                        from setproctitle import setproctitle  # type: ignore[import]
-
-                        setproctitle(
-                            "{python} {pex_file} {args}".format(
-                                python=sys.executable,
-                                pex_file=pex_file,
-                                args=" ".join(sys.argv[1:]),
-                            )
-                        )
-                    except ImportError:
-                        TRACER.log(
-                            "Not setting process title since setproctitle is not available in "
-                            "{pex_file}".format(pex_file=pex_file),
-                            V=3,
-                        )
-
-                exit_value = self._wrap_coverage(self._wrap_profiling, self._execute)
-            sys.exit(exit_value)
-        except Exception:
-            # Allow the current sys.excepthook to handle this app exception before we tear things
-            # down in finally, then reraise so that the exit status is reflected correctly.
-            sys.excepthook(*sys.exc_info())
-            raise
-        except SystemExit as se:
-            # Print a SystemExit error message, avoiding a traceback in python3.
-            # This must happen here, as sys.stderr is about to be torn down
-            if not isinstance(se.code, int) and se.code is not None:  # type: ignore[unreachable]
-                print(se.code, file=sys.stderr)  # type: ignore[unreachable]
-            raise
-        finally:
-            # squash all exceptions on interpreter teardown -- the primary type here are
-            # atexit handlers failing to run because of things such as:
-            #   http://stackoverflow.com/questions/2572172/referencing-other-modules-in-atexit
-            if not teardown_verbosity:
-                sys.stderr.flush()
-                sys.stderr = open(os.devnull, "w")
-                if PY3:
-                    # Python 3 warns about unclosed resources. In this case we intentionally do not
-                    # close `/dev/null` since we want all stderr to flow there until the latest
-                    # stages of Python interpreter shutdown when the Python runtime will `del` the
-                    # open file and thus finally close the underlying file descriptor. As such,
-                    # suppress the warning.
-                    warnings.filterwarnings(
-                        action="ignore",
-                        message=r"unclosed file {}".format(re.escape(str(sys.stderr))),
-                        category=ResourceWarning,
-                    )
-                sys.excepthook = lambda *a, **kw: None
+        result = self._wrap_coverage(self._wrap_profiling, self._execute)
+        if "PYTHONINSPECT" not in os.environ:
+            sys.exit(0 if isinstance(result, Globals) else result)
+        else:
+            return result
 
     def _execute(self):
         # type: () -> Any
@@ -532,10 +588,24 @@ class PEX(object):  # noqa: T000
 
         self._clean_environment(strip_pex_env=self._pex_info.strip_pex_env)
 
+        for name, value in self._pex_info.inject_env.items():
+            os.environ.setdefault(name, value)
+
         if force_interpreter:
             TRACER.log("PEX_INTERPRETER specified, dropping into interpreter")
             return self.execute_interpreter()
 
+        if not any(
+            (
+                self._pex_info_overrides.script,
+                self._pex_info_overrides.entry_point,
+                self._pex_info.script,
+                self._pex_info.entry_point,
+            )
+        ):
+            TRACER.log("No entry point specified, dropping into interpreter")
+            return self.execute_interpreter()
+
         if self._pex_info_overrides.script and self._pex_info_overrides.entry_point:
             return "Cannot specify both script and entry_point for a PEX!"
 
@@ -544,48 +614,43 @@ class PEX(object):  # noqa: T000
 
         if self._pex_info_overrides.script:
             return self.execute_script(self._pex_info_overrides.script)
-        elif self._pex_info_overrides.entry_point:
-            return self.execute_entry(self._pex_info_overrides.entry_point)
-        elif self._pex_info.script:
+        if self._pex_info_overrides.entry_point:
+            return self.execute_entry(parse_entry_point(self._pex_info_overrides.entry_point))
+
+        sys.argv[1:1] = list(self._pex_info.inject_args)
+
+        if self._pex_info.script:
             return self.execute_script(self._pex_info.script)
-        elif self._pex_info.entry_point:
-            return self.execute_entry(self._pex_info.entry_point)
         else:
-            TRACER.log("No entry point specified, dropping into interpreter")
-            return self.execute_interpreter()
-
-    @classmethod
-    def demote_bootstrap(cls):
-        TRACER.log("Bootstrap complete, performing final sys.path modifications...")
-
-        should_log = {level: TRACER.should_log(V=level) for level in range(1, 10)}
-
-        def log(msg, V=1):
-            if should_log.get(V, False):
-                print("pex: {}".format(msg), file=sys.stderr)
-
-        # Remove the third party resources pex uses and demote pex bootstrap code to the end of
-        # sys.path for the duration of the run to allow conflicting versions supplied by user
-        # dependencies to win during the course of the execution of user code.
-        third_party.uninstall()
-
-        bootstrap = Bootstrap.locate()
-        log("Demoting code from %s" % bootstrap, V=2)
-        for module in bootstrap.demote():
-            log("un-imported {}".format(module), V=9)
-
-        import pex
-
-        log("Re-imported pex from {}".format(pex.__path__), V=3)
-
-        log("PYTHONPATH contains:")
-        for element in sys.path:
-            log("  %c %s" % (" " if os.path.exists(element) else "*", element))
-        log("  * - paths that do not exist or will be imported via zipimport")
+            # N.B.: We tested one of script or entry_point in pex_info or overrides was non-empty
+            # above; so this just satisfies type checking.
+            assert self._pex_info.entry_point
+            return self.execute_entry(parse_entry_point(self._pex_info.entry_point))
 
     def execute_interpreter(self):
         # type: () -> Any
+
+        # A Python interpreter always inserts the CWD at the head of the sys.path.
+        # See https://docs.python.org/3/library/sys.html#sys.path
+        sys.path.insert(0, "")
+
         args = sys.argv[1:]
+        python_options = []
+        for index, arg in enumerate(args):
+            # Check if the arg is an expected startup arg.
+            if arg.startswith("-") and arg not in ("-", "-c", "-m"):
+                python_options.append(arg)
+            else:
+                args = args[index:]
+                break
+        else:
+            # All the args were python options
+            args = []
+
+        # The pex was called with Python interpreter options
+        if python_options:
+            return self.execute_with_options(python_options, args)
+
         if args:
             # NB: We take care here to setup sys.argv to match how CPython does it for each case.
             arg = args[0]
@@ -596,13 +661,14 @@ class PEX(object):  # noqa: T000
             elif arg == "-m":
                 module = args[1]
                 sys.argv = args[1:]
-                return self.execute_module(module, alter_sys=True)
+                return self.execute_module(module)
             else:
                 try:
                     if arg == "-":
                         content = sys.stdin.read()
                     else:
-                        with open(arg) as fp:
+                        file_path = arg if os.path.isfile(arg) else os.path.join(arg, "__main__.py")
+                        with open(file_path) as fp:
                             content = fp.read()
                 except IOError as e:
                     return "Could not open {} in the environment [{}]: {}".format(
@@ -611,21 +677,67 @@ class PEX(object):  # noqa: T000
                 sys.argv = args
                 return self.execute_content(arg, content)
         else:
-            self.demote_bootstrap()
+            pex_repl = repl.create_pex_repl(
+                pex_info=self.pex_info(),
+                requirements=(
+                    tuple(
+                        OrderedSet(
+                            itertools.chain.from_iterable(
+                                env.pex_info.requirements for env in self._envs
+                            )
+                        )
+                    )
+                    if self._envs
+                    else ()
+                ),
+                activated_dists=self._activated_dists or (),
+            )
 
-            import code
+            bootstrap.demote()
 
-            code.interact()
-            return None
+            return Globals(pex_repl())
+
+    @staticmethod
+    def execute_with_options(
+        python_options,  # type: List[str]
+        args,  # List[str]
+    ):
+        # type: (...) -> Union[NoReturn, Any]
+        """
+        Restart the process passing the given options to the python interpreter
+        """
+        # Find the installed (unzipped) PEX entry point.
+        main = sys.modules.get("__main__")
+        if not main or not main.__file__:
+            # N.B.: This should never happen.
+            return "Unable to resolve PEX __main__ module file: {}".format(main)
+
+        python = sys.executable
+        cmdline = [python] + python_options + [main.__file__] + args
+        TRACER.log(
+            "Re-executing with Python interpreter options: cmdline={cmdline!r}".format(
+                cmdline=" ".join(cmdline)
+            )
+        )
+        if any(
+            arg.startswith("-") and not arg.startswith("--") and "i" in arg
+            for arg in python_options
+        ):
+            os.environ["PYTHONINSPECT"] = "1"
+        safe_execv(cmdline)
 
     def execute_script(self, script_name):
         # type: (str) -> Any
         dists = list(self.activate())
 
-        dist, entry_point = get_entry_point_from_console_script(script_name, dists)
-        if entry_point:
-            TRACER.log("Found console_script {!r} in {!r}.".format(entry_point, dist))
-            return self.execute_entry(entry_point)
+        dist_entry_point = get_entry_point_from_console_script(script_name, dists)
+        if dist_entry_point:
+            TRACER.log(
+                "Found {console_script}.".format(
+                    console_script=dist_entry_point.render_description()
+                )
+            )
+            return self.execute_entry(dist_entry_point.entry_point)
 
         dist_script = get_script_from_distributions(script_name, dists)
         if not dist_script:
@@ -634,9 +746,7 @@ class PEX(object):  # noqa: T000
         TRACER.log("Found script {!r} in {!r}.".format(script_name, dist_script.dist))
         ast = dist_script.python_script()
         if ast:
-            return self.execute_ast(
-                dist_script.path, dist_script.read_contents(), argv0=script_name
-            )
+            return self.execute_ast(dist_script.path, ast, argv0=script_name)
         else:
             return self.execute_external(dist_script.path)
 
@@ -656,7 +766,7 @@ class PEX(object):  # noqa: T000
         content,  # type: str
         argv0=None,  # type: Optional[str]
     ):
-        # type: (...) -> Optional[str]
+        # type: (...) -> Any
         try:
             program = compile(content, name, "exec", flags=0, dont_inherit=1)
         except SyntaxError as e:
@@ -670,8 +780,8 @@ class PEX(object):  # noqa: T000
         program,  # type: ast.AST
         argv0=None,  # type: Optional[str]
     ):
-        # type: (...) -> Optional[str]
-        cls.demote_bootstrap()
+        # type: (...) -> Any
+        bootstrap.demote()
 
         from pex.compatibility import exec_function
 
@@ -679,44 +789,29 @@ class PEX(object):  # noqa: T000
         globals_map = globals().copy()
         globals_map["__name__"] = "__main__"
         globals_map["__file__"] = name
-        exec_function(program, globals_map)
-        return None
+        return Globals(exec_function(program, globals_map))
 
     def execute_entry(self, entry_point):
+        # type: (Union[ModuleEntryPoint, CallableEntryPoint]) -> Any
+        if isinstance(entry_point, CallableEntryPoint):
+            return self.execute_entry_point(entry_point)
+
+        return self.execute_module(entry_point.module)
+
+    def execute_module(self, module_name):
         # type: (str) -> Any
-        if ":" in entry_point:
-            return self.execute_pkg_resources(entry_point)
-
-        # When running as a zipapp we can't usefully `alter_sys` to point to the module we're
-        # executing as argv[0] since that module will be contained in a zipfile. In other words, if
-        # we did do this, sys.argv[0] would be `/path/to/pex.zip/path/to/module.py` and that value
-        # would not be usable in the standard way. Its not a file you can read or re-execute
-        # against like a loose python module source file would be.
-        #
-        # Python itself disallows this case altogether in the standard zipapp module (probably for
-        # similar reasons). See: https://docs.python.org/3/library/zipapp.html#cmdoption-zipapp-m
-        alter_sys = not zipfile.is_zipfile(self._pex)
-        return self.execute_module(entry_point, alter_sys)
-
-    def execute_module(
-        self,
-        module_name,  # type: str
-        alter_sys,  # type: bool
-    ):
-        # type: (...) -> None
-        self.demote_bootstrap()
+        bootstrap.demote()
 
         import runpy
 
-        runpy.run_module(module_name, run_name="__main__", alter_sys=alter_sys)
+        return Globals(runpy.run_module(module_name, run_name="__main__", alter_sys=True))
 
     @classmethod
-    def execute_pkg_resources(cls, spec):
-        # type: (str) -> Any
-        entry = EntryPoint.parse("run = {}".format(spec))
-        cls.demote_bootstrap()
+    def execute_entry_point(cls, entry_point):
+        # type: (CallableEntryPoint) -> Any
+        bootstrap.demote()
 
-        runner = entry.resolve()
+        runner = entry_point.resolve()
         return runner()
 
     def cmdline(self, args=()):
@@ -748,11 +843,12 @@ class PEX(object):  # noqa: T000
             env = os.environ.copy()
             self._clean_environment(env=env)
 
+        kwargs = dict(subprocess_daemon_kwargs() if setsid else {}, **kwargs)
+
         TRACER.log("PEX.run invoking {}".format(" ".join(self.cmdline(args))))
         _, process = self._interpreter.open_process(
             [self._pex] + list(args),
             cwd=self._pex if with_chroot else os.getcwd(),
-            preexec_fn=os.setsid if setsid else None,
             stdin=kwargs.pop("stdin", None),
             stdout=kwargs.pop("stdout", None),
             stderr=kwargs.pop("stderr", None),
