@@ -21,21 +21,21 @@ from pex.common import safe_mkdir, safe_mkdtemp
 from pex.compatibility import get_stderr_bytes_buffer, shlex_quote, urlparse
 from pex.dependency_configuration import DependencyConfiguration
 from pex.dist_metadata import Requirement
+from pex.fetcher import URLFetcher
 from pex.interpreter import PythonInterpreter
 from pex.jobs import Job
 from pex.network_configuration import NetworkConfiguration
 from pex.pep_427 import install_wheel_interpreter
-from pex.pip import dependencies, foreign_platform
+from pex.pep_508 import MarkerEnvironment
+from pex.pip import dependencies, foreign_platform, package_repositories
 from pex.pip.download_observer import DownloadObserver, PatchSet
 from pex.pip.log_analyzer import ErrorAnalyzer, ErrorMessage, LogAnalyzer, LogScrapeJob
 from pex.pip.tailer import Tailer
 from pex.pip.version import PipVersion, PipVersionValue
 from pex.platforms import PlatformSpec
-from pex.resolve.resolver_configuration import (
-    BuildConfiguration,
-    ReposConfiguration,
-    ResolverVersion,
-)
+from pex.resolve.package_repository import ReposConfiguration
+from pex.resolve.resolver_configuration import BuildConfiguration, ResolverVersion
+from pex.resolve.target_system import UniversalTarget
 from pex.targets import Target
 from pex.tracer import TRACER
 from pex.typing import TYPE_CHECKING
@@ -54,7 +54,9 @@ if TYPE_CHECKING:
         Match,
         Optional,
         Sequence,
+        Text,
         Tuple,
+        Union,
     )
 
     import attr  # vendor:skip
@@ -64,8 +66,8 @@ else:
 
 @attr.s(frozen=True)
 class PipArgs(object):
-    indexes = attr.ib(default=None)  # type: Optional[Sequence[str]]
-    find_links = attr.ib(default=None)  # type: Optional[Iterable[str]]
+    indexes = attr.ib(default=None)  # type: Optional[Sequence[Text]]
+    find_links = attr.ib(default=None)  # type: Optional[Iterable[Text]]
     network_configuration = attr.ib(default=None)  # type: Optional[NetworkConfiguration]
 
     def iter(self, version):
@@ -152,10 +154,8 @@ class PackageIndexConfiguration(object):
         cls,
         pip_version=None,  # type: Optional[PipVersionValue]
         resolver_version=None,  # type: Optional[ResolverVersion.Value]
-        indexes=None,  # type: Optional[Sequence[str]]
-        find_links=None,  # type: Optional[Iterable[str]]
+        repos_configuration=ReposConfiguration(),  # type: ReposConfiguration
         network_configuration=None,  # type: Optional[NetworkConfiguration]
-        password_entries=(),  # type: Iterable[PasswordEntry]
         use_pip_config=False,  # type: bool
         extra_pip_requirements=(),  # type: Tuple[Requirement, ...]
         keyring_provider=None,  # type: Optional[str]
@@ -174,8 +174,8 @@ class PackageIndexConfiguration(object):
             resolver_version=resolver_version,
             network_configuration=network_configuration,
             args=PipArgs(
-                indexes=indexes,
-                find_links=find_links,
+                indexes=repos_configuration.indexes,
+                find_links=repos_configuration.find_links,
                 network_configuration=network_configuration,
             ),
             env=cls._calculate_env(
@@ -183,7 +183,7 @@ class PackageIndexConfiguration(object):
             ),
             use_pip_config=use_pip_config,
             extra_pip_requirements=extra_pip_requirements,
-            password_entries=password_entries,
+            repos_configuration=repos_configuration,
             keyring_provider=keyring_provider,
         )
 
@@ -194,7 +194,7 @@ class PackageIndexConfiguration(object):
         args,  # type: PipArgs
         env,  # type: Iterable[Tuple[str, str]]
         use_pip_config,  # type: bool
-        password_entries=(),  # type: Iterable[PasswordEntry]
+        repos_configuration=ReposConfiguration(),  # type: ReposConfiguration
         pip_version=None,  # type: Optional[PipVersionValue]
         extra_pip_requirements=(),  # type: Tuple[Requirement, ...]
         keyring_provider=None,  # type: Optional[str]
@@ -205,11 +205,32 @@ class PackageIndexConfiguration(object):
         self.args = args  # type: PipArgs
         self.env = dict(env)  # type: Mapping[str, str]
         self.use_pip_config = use_pip_config  # type: bool
-        self.password_entries = password_entries  # type: Iterable[PasswordEntry]
+        self.repos_configuration = repos_configuration
         self.pip_version = pip_version  # type: Optional[PipVersionValue]
         self.extra_pip_requirements = extra_pip_requirements  # type: Tuple[Requirement, ...]
         self.keyring_provider = keyring_provider  # type: Optional[str]
 
+    @property
+    def password_entries(self):
+        # type: () -> Iterable[PasswordEntry]
+        return self.repos_configuration.password_entries
+
+    def patch(
+        self,
+        pip_version,  # type: PipVersionValue
+        target,  # type:  Union[UniversalTarget, MarkerEnvironment]
+        requirement_files=None,  # type: Optional[Iterable[str]]
+    ):
+        # type: (...) -> Optional[DownloadObserver]
+        return package_repositories.patch(
+            repos_configuration=self.repos_configuration.with_contained_repos(
+                requirement_files,
+                fetcher=URLFetcher(network_configuration=self.network_configuration),
+            ),
+            pip_version=pip_version,
+            target=target,
+        )
+
 
 if TYPE_CHECKING:
     from pex.pip.log_analyzer import ErrorAnalysis
@@ -455,7 +476,7 @@ class Pip(object):
 
         command = pip_args + list(args)
 
-        # N.B.: Package index options in Pep always have the same option names, but they are
+        # N.B.: Package index options in Pip always have the same option names, but they are
         # registered as subcommand-specific, so we must append them here _after_ the pip subcommand
         # specified in `args`.
         if package_index_configuration:
@@ -576,6 +597,7 @@ class Pip(object):
         build_configuration=BuildConfiguration(),  # type: BuildConfiguration
         observer=None,  # type: Optional[DownloadObserver]
         dependency_configuration=DependencyConfiguration(),  # type: DependencyConfiguration
+        universal_target=None,  # type: Optional[UniversalTarget]
         log=None,  # type: Optional[str]
     ):
         # type: (...) -> Job
@@ -623,7 +645,18 @@ class Pip(object):
         for obs in (
             foreign_platform_observer,
             observer,
-            dependencies.patch(dependency_configuration),
+            dependencies.patch(
+                dependency_configuration, target=universal_target or target.marker_environment
+            ),
+            (
+                package_index_configuration.patch(
+                    pip_version=self.version,
+                    target=universal_target or target.marker_environment,
+                    requirement_files=requirement_files,
+                )
+                if package_index_configuration
+                else None
+            ),
         ):
             if obs:
                 if obs.analyzer:
@@ -717,7 +750,7 @@ class Pip(object):
                     build_configuration=BuildConfiguration.create(allow_builds=False),
                 ).wait()
                 for wheel in glob.glob(os.path.join(atomic_dir.work_dir, "*.whl")):
-                    install_wheel_interpreter(wheel_path=wheel, interpreter=pip_interpreter)
+                    install_wheel_interpreter(wheel=wheel, interpreter=pip_interpreter)
 
     def spawn_build_wheels(
         self,
