@@ -24,11 +24,12 @@ from pex.pip.log_analyzer import LogAnalyzer
 from pex.pip.vcs import fingerprint_downloaded_vcs_archive
 from pex.pip.version import PipVersionValue
 from pex.requirements import LocalProjectRequirement, VCSRequirement
-from pex.resolve.locked_resolve import LockConfiguration, LockStyle, TargetSystem
+from pex.resolve.locked_resolve import LockStyle
 from pex.resolve.pep_691.fingerprint_service import FingerprintService
 from pex.resolve.pep_691.model import Endpoint
 from pex.resolve.resolved_requirement import PartialArtifact, Pin, ResolvedRequirement
 from pex.resolve.resolvers import Resolver
+from pex.resolve.target_system import TargetSystem, UniversalTarget
 from pex.targets import Target
 from pex.typing import TYPE_CHECKING
 
@@ -230,7 +231,7 @@ class Locker(LogAnalyzer):
         target,  # type: Target
         root_requirements,  # type: Iterable[ParsedRequirement]
         resolver,  # type: Resolver
-        lock_configuration,  # type: LockConfiguration
+        lock_style,  # type: LockStyle.Value
         download_dir,  # type: str
         fingerprint_service=None,  # type: Optional[FingerprintService]
         pip_version=None,  # type: Optional[PipVersionValue]
@@ -241,7 +242,7 @@ class Locker(LogAnalyzer):
         self._vcs_url_manager = VCSURLManager.create(root_requirements)
         self._pip_version = pip_version
         self._resolver = resolver
-        self._lock_configuration = lock_configuration
+        self._lock_style = lock_style
         self._download_dir = download_dir
         self._fingerprint_service = fingerprint_service or FingerprintService()
 
@@ -264,16 +265,6 @@ class Locker(LogAnalyzer):
         self._local_projects = OrderedSet()  # type: OrderedSet[str]
         self._lock_result = None  # type: Optional[LockResult]
 
-    @property
-    def style(self):
-        # type: () -> LockStyle.Value
-        return self._lock_configuration.style
-
-    @property
-    def requires_python(self):
-        # type: () -> Tuple[str, ...]
-        return self._lock_configuration.requires_python
-
     def should_collect(self, returncode):
         # type: (int) -> bool
         return returncode == 0
@@ -388,8 +379,9 @@ class Locker(LogAnalyzer):
                     )
                     self._selected_path_to_pin[selected_path] = build_result.pin
 
-                    vcs, _, vcs_url = artifact_url.normalized_url.partition("+")
-                    vcs_url, _, _ = vcs_url.partition("@")
+                    vcs, _, vcs_url = build_result.url.raw_url.partition("+")
+                    if "@" in build_result.url.path:
+                        vcs_url, _, _ = vcs_url.rpartition("@")
                     commit_id = self._commit_ids.pop(vcs_url, None)
                 elif isinstance(artifact_url.scheme, ArchiveScheme.Value):
                     selected_path = os.path.basename(artifact_url.path)
@@ -523,7 +515,7 @@ class Locker(LogAnalyzer):
                 self._saved.add(build_result_pin)
             return self.Continue()
 
-        if self.style in (LockStyle.SOURCES, LockStyle.UNIVERSAL):
+        if self._lock_style in (LockStyle.SOURCES, LockStyle.UNIVERSAL):
             match = re.search(r"Found link (?P<url>\S+)(?: \(from .*\))?, version: ", line)
             if match:
                 url = self.parse_url_and_maybe_record_fingerprint(match.group("url"))
@@ -611,42 +603,39 @@ _PLATFORM_TAG_REGEXP = {
 }
 
 
-def patch(lock_configuration):
-    # type: (LockConfiguration) -> PatchSet
+def patch(universal_target):
+    # type: (Optional[UniversalTarget]) -> PatchSet
 
-    if lock_configuration.style != LockStyle.UNIVERSAL:
+    if not universal_target:
         return PatchSet()
 
     patches_dir = safe_mkdtemp()
     patches = []
-    if lock_configuration.requires_python:
+    if universal_target.requires_python:
         patches.append(
             foreign_platform.patch_requires_python(
-                requires_python=lock_configuration.requires_python, patches_dir=patches_dir
+                requires_python=universal_target.requires_python,
+                patches_dir=patches_dir,
             )
         )
 
     env = {}  # type: Dict[str, str]
-    if lock_configuration.target_systems and set(lock_configuration.target_systems) != set(
-        TargetSystem.values()
-    ):
+    if universal_target.implementation:
+        env.update(_PEX_INTERPRETER_IMPLEMENTATION=str(universal_target.implementation))
+
+    if universal_target.systems and set(universal_target.systems) != set(TargetSystem.values()):
         target_systems = {
-            "os_names": [
-                _OS_NAME[target_system] for target_system in lock_configuration.target_systems
-            ],
+            "os_names": [_OS_NAME[target_system] for target_system in universal_target.systems],
             "platform_systems": [
-                _PLATFORM_SYSTEM[target_system]
-                for target_system in lock_configuration.target_systems
+                _PLATFORM_SYSTEM[target_system] for target_system in universal_target.systems
             ],
             "sys_platforms": list(
                 itertools.chain.from_iterable(
-                    _SYS_PLATFORMS[target_system]
-                    for target_system in lock_configuration.target_systems
+                    _SYS_PLATFORMS[target_system] for target_system in universal_target.systems
                 )
             ),
             "platform_tag_regexps": [
-                _PLATFORM_TAG_REGEXP[target_system]
-                for target_system in lock_configuration.target_systems
+                _PLATFORM_TAG_REGEXP[target_system] for target_system in universal_target.systems
             ],
         }
         with open(os.path.join(patches_dir, "target_systems.json"), "w") as fp:
