@@ -5,8 +5,9 @@ from __future__ import absolute_import
 
 import os
 
-from pex.dist_metadata import Distribution, Requirement
-from pex.interpreter import PythonInterpreter, calculate_binary_name
+from pex.dist_metadata import Constraint, Distribution
+from pex.interpreter import PythonInterpreter
+from pex.interpreter_implementation import InterpreterImplementation
 from pex.orderedset import OrderedSet
 from pex.pep_425 import CompatibilityTags, RankedTag
 from pex.pep_508 import MarkerEnvironment
@@ -48,14 +49,25 @@ class Target(object):
     id = attr.ib()  # type: str
     platform = attr.ib()  # type: Platform
     marker_environment = attr.ib()  # type: MarkerEnvironment
+    implementation = attr.ib(init=False)  # type: Optional[InterpreterImplementation.Value]
+
+    def __attrs_post_init__(self):
+        interpreter_implementation = None  # type: Optional[InterpreterImplementation.Value]
+        for interpreter_impl in InterpreterImplementation.values():
+            if interpreter_impl.value == self.marker_environment.platform_python_implementation:
+                interpreter_implementation = interpreter_impl
+                break
+        object.__setattr__(self, "implementation", interpreter_implementation)
 
     def binary_name(self, version_components=2):
         # type: (int) -> str
-        return calculate_binary_name(
-            platform_python_implementation=self.marker_environment.platform_python_implementation,
-            python_version=self.python_version[:version_components]
-            if self.python_version and version_components > 0
-            else None,
+        interpreter_implementation = self.implementation or InterpreterImplementation.CPYTHON
+        return interpreter_implementation.calculate_binary_name(
+            version=(
+                self.python_version[:version_components]
+                if self.python_version and version_components > 0
+                else None
+            )
         )
 
     @property
@@ -145,7 +157,7 @@ class Target(object):
 
     def requirement_applies(
         self,
-        requirement,  # type: Requirement
+        requirement,  # type: Constraint
         extras=(),  # type: Iterable[str]
     ):
         # type: (...) -> bool
@@ -171,7 +183,7 @@ class Target(object):
 
     def wheel_applies(self, wheel):
         # type: (Distribution) -> WheelEvaluation
-        wheel_tags = CompatibilityTags.from_wheel(wheel.location)
+        wheel_tags = CompatibilityTags.from_wheel(wheel)
         ranked_tag = self.supported_tags.best_match(wheel_tags)
         return WheelEvaluation(
             tags=tuple(wheel_tags),
@@ -193,6 +205,7 @@ class Target(object):
         return str(self.platform.tag)
 
     def render_description(self):
+        # type: () -> str
         raise NotImplementedError()
 
     def __repr__(self):
@@ -247,6 +260,7 @@ class LocalInterpreter(Target):
         return self.interpreter.binary
 
     def render_description(self):
+        # type: () -> str
         return "{platform} interpreter at {path}".format(
             platform=self.interpreter.platform.tag, path=self.interpreter.binary
         )
@@ -269,6 +283,7 @@ class AbbreviatedPlatform(Target):
         return self.platform.supported_tags
 
     def render_description(self):
+        # type: () -> str
         return "abbreviated platform {platform}".format(platform=self.platform.tag)
 
 
@@ -311,6 +326,7 @@ class CompletePlatform(Target):
         return self._supported_tags
 
     def render_description(self):
+        # type: () -> str
         return "complete platform {platform}".format(platform=self.platform.tag)
 
 
@@ -330,6 +346,11 @@ class Targets(object):
     complete_platforms = attr.ib(default=())  # type: Tuple[CompletePlatform, ...]
     platforms = attr.ib(default=())  # type: Tuple[Optional[Platform], ...]
 
+    @property
+    def is_empty(self):
+        # type: () -> bool
+        return not self.interpreters and not self.complete_platforms and not self.platforms
+
     @property
     def interpreter(self):
         # type: () -> Optional[PythonInterpreter]
@@ -391,7 +412,7 @@ class Targets(object):
 
     def require_at_most_one_target(self, purpose):
         # type: (str) -> Union[Optional[Target], Error]
-        resolved_targets = self.unique_targets(only_explicit=False)
+        resolved_targets = self.unique_targets(only_explicit=True)
         if len(resolved_targets) > 1:
             return Error(
                 "At most a single target is required for {purpose}.\n"
