@@ -1,14 +1,13 @@
-# Copyright 2021 Pants project contributors (see CONTRIBUTORS.md).
+# Copyright 2021 Pex project contributors.
 # Licensed under the Apache License, Version 2.0 (see LICENSE).
 import os
 import shutil
-import subprocess
 import sys
 
 from pex.pex_info import PexInfo
-from pex.testing import run_pex_command
 from pex.typing import TYPE_CHECKING
 from pex.variables import unzip_dir
+from testing import run_pex_command, subprocess
 
 if TYPE_CHECKING:
     from typing import Any
