@@ -1,15 +1,15 @@
-# Copyright 2021 Pants project contributors (see CONTRIBUTORS.md).
+# Copyright 2021 Pex project contributors.
 # Licensed under the Apache License, Version 2.0 (see LICENSE).
 
 import os
 
 from pex.common import temporary_dir
-from pex.testing import PY310, ensure_python_interpreter, run_pex_command, run_simple_pex
+from testing import PY310, ensure_python_interpreter, run_pex_command, run_simple_pex
 
 
 def test_entry_point_targeting():
     # type: () -> None
-    """Test bugfix for https://github.com/pantsbuild/pex/issues/434."""
+    """Test bugfix for https://github.com/pex-tool/pex/issues/434."""
     with temporary_dir() as td:
         pexrc_path = os.path.join(td, ".pexrc")
         with open(pexrc_path, "w") as pexrc:
