@@ -1,35 +1,42 @@
-# Copyright 2022 Pants project contributors (see CONTRIBUTORS.md).
+# Copyright 2022 Pex project contributors.
 # Licensed under the Apache License, Version 2.0 (see LICENSE).
+
 from textwrap import dedent
 
 import pytest
 
 from pex import targets
+from pex.artifact_url import VCS, ArtifactURL, Fingerprint
+from pex.dist_metadata import Requirement
 from pex.interpreter import PythonInterpreter
-from pex.pep_425 import TagRank
+from pex.pep_425 import CompatibilityTags, TagRank
 from pex.pep_440 import Version
 from pex.pep_503 import ProjectName
-from pex.platforms import Platform
+from pex.pep_508 import MarkerEnvironment
+from pex.resolve import abbreviated_platforms
 from pex.resolve.locked_resolve import (
     Artifact,
     DownloadableArtifact,
+    FileArtifact,
+    LocalProjectArtifact,
     LockedRequirement,
     LockedResolve,
     RankedArtifact,
     Resolved,
+    VCSArtifact,
     _ResolvedArtifact,
 )
-from pex.resolve.resolved_requirement import Fingerprint, Pin
+from pex.resolve.resolved_requirement import PartialArtifact, Pin
+from pex.resolve.resolver_configuration import BuildConfiguration
 from pex.result import Error, try_
 from pex.sorted_tuple import SortedTuple
-from pex.targets import AbbreviatedPlatform, LocalInterpreter, Target
+from pex.targets import AbbreviatedPlatform, CompletePlatform, LocalInterpreter, Target
 from pex.third_party.packaging.specifiers import SpecifierSet
 from pex.third_party.packaging.tags import Tag
-from pex.third_party.pkg_resources import Requirement
 from pex.typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
-    from typing import Iterable, Union
+    from typing import Iterable, Iterator, Union
 
 
 @pytest.fixture
@@ -39,9 +46,9 @@ def current_target():
 
 
 @pytest.fixture
-def py37_target(py37):
+def py39_target(py39):
     # type: (PythonInterpreter) -> Target
-    return LocalInterpreter.create(py37)
+    return LocalInterpreter.create(py39)
 
 
 @pytest.fixture
@@ -63,8 +70,8 @@ def artifact(
     algorithm,  # type: str
     hash,  # type: str
 ):
-    # type: (...) -> Artifact
-    return Artifact(url=url, fingerprint=Fingerprint(algorithm=algorithm, hash=hash))
+    # type: (...) -> Union[FileArtifact, LocalProjectArtifact, VCSArtifact]
+    return Artifact.from_url(url=url, fingerprint=Fingerprint(algorithm=algorithm, hash=hash))
 
 
 def locked_requirements(*locked_requirements):
@@ -91,7 +98,7 @@ def ansicolors_simple():
             LockedRequirement.create(
                 pin=pin("ansicolors", "1.1.7"),
                 artifact=artifact(
-                    url="https://example.org/ansicolors-1.1.7-py2.py3-none-any.whl",
+                    url="https://example.org/ansicolors-1.1.7-1buildtag-py2.py3-none-any.whl",
                     algorithm="blake256",
                     hash="cafebabe",
                 ),
@@ -116,7 +123,7 @@ def ansicolors_simple():
 
 
 def assert_resolved(
-    result,  # type: Union[Resolved, Error]
+    result,  # type: Union[Resolved[LockedResolve], Error]
     *downloadable_artifacts  # type: DownloadableArtifact
 ):
     # type: (...) -> None
@@ -141,11 +148,15 @@ def test_build(
         ),
     )
     assert_resolved(
-        ansicolors_simple.resolve(current_target, [req("ansicolors")], build=False),
+        ansicolors_simple.resolve(
+            current_target,
+            [req("ansicolors")],
+            build_configuration=BuildConfiguration.create(allow_builds=False),
+        ),
         DownloadableArtifact.create(
             pin=pin("ansicolors", "1.1.7"),
             artifact=artifact(
-                url="https://example.org/ansicolors-1.1.7-py2.py3-none-any.whl",
+                url="https://example.org/ansicolors-1.1.7-1buildtag-py2.py3-none-any.whl",
                 algorithm="blake256",
                 hash="cafebabe",
             ),
@@ -164,7 +175,7 @@ def test_use_wheel(
         DownloadableArtifact.create(
             pin=pin("ansicolors", "1.1.7"),
             artifact=artifact(
-                url="https://example.org/ansicolors-1.1.7-py2.py3-none-any.whl",
+                url="https://example.org/ansicolors-1.1.7-1buildtag-py2.py3-none-any.whl",
                 algorithm="blake256",
                 hash="cafebabe",
             ),
@@ -172,7 +183,11 @@ def test_use_wheel(
         ),
     )
     assert_resolved(
-        ansicolors_simple.resolve(current_target, [req("ansicolors==1.1.7")], use_wheel=False),
+        ansicolors_simple.resolve(
+            current_target,
+            [req("ansicolors==1.1.7")],
+            build_configuration=BuildConfiguration.create(allow_wheels=False),
+        ),
         DownloadableArtifact.create(
             pin=pin("ansicolors", "1.1.7"),
             artifact=artifact(
@@ -185,6 +200,39 @@ def test_use_wheel(
     )
 
 
+def test_constraints(
+    current_target,  # type: Target
+    ansicolors_simple,  # type: LockedResolve
+):
+    # type: (...) -> None
+    assert_resolved(
+        ansicolors_simple.resolve(current_target, [req("ansicolors")]),
+        DownloadableArtifact.create(
+            pin=pin("ansicolors", "1.1.8"),
+            artifact=artifact(
+                url="https://example.org/ansicolors-1.1.8.tar.gz",
+                algorithm="md5",
+                hash="cafebabe",
+            ),
+            satisfied_direct_requirements=requirements("ansicolors"),
+        ),
+    )
+    assert_resolved(
+        ansicolors_simple.resolve(
+            current_target, [req("ansicolors")], constraints=[req("ansicolors<1.1.8")]
+        ),
+        DownloadableArtifact.create(
+            pin=pin("ansicolors", "1.1.7"),
+            artifact=artifact(
+                url="https://example.org/ansicolors-1.1.7-1buildtag-py2.py3-none-any.whl",
+                algorithm="blake256",
+                hash="cafebabe",
+            ),
+            satisfied_direct_requirements=requirements("ansicolors"),
+        ),
+    )
+
+
 @pytest.fixture
 def ansicolors_exotic():
     # type: () -> LockedResolve
@@ -216,7 +264,7 @@ def ansicolors_exotic():
 
 
 def assert_error(
-    result,  # type: Union[Resolved, Error]
+    result,  # type: Union[Resolved[LockedResolve], Error]
     expected_error_message,  # type: str
 ):
     assert Error(expected_error_message.strip()) == result
@@ -224,28 +272,7 @@ def assert_error(
 
 def platform(plat):
     # type: (str) -> AbbreviatedPlatform
-    return AbbreviatedPlatform.create(Platform.create(plat))
-
-
-def test_invalid_configuration(
-    current_target,  # type: Target
-    ansicolors_exotic,  # type: LockedResolve
-):
-    # type: (...) -> None
-    assert_error(
-        ansicolors_exotic.resolve(
-            current_target, [req("ansicolors")], build=False, use_wheel=False
-        ),
-        "Cannot both ignore wheels (use_wheel=False) and refrain from building distributions "
-        "(build=False).",
-    )
-
-    platform_target = platform("linux-x86_64-cp-37-m")
-    assert_error(
-        ansicolors_exotic.resolve(platform_target, [req("ansicolors")], use_wheel=False),
-        "Cannot ignore wheels (use_wheel=False) when resolving for a platform: given "
-        "{target_description}".format(target_description=platform_target.render_description()),
-    )
+    return AbbreviatedPlatform.create(abbreviated_platforms.create(plat))
 
 
 def test_platform_resolve(ansicolors_exotic):
@@ -370,7 +397,11 @@ def test_wheel_tag_mismatch(
 ):
     # type: (...) -> None
     assert_error(
-        ansicolors_exotic.resolve(current_target, [req("ansicolors==1.1.*")], build=False),
+        ansicolors_exotic.resolve(
+            current_target,
+            [req("ansicolors==1.1.*")],
+            build_configuration=BuildConfiguration.create(allow_builds=False),
+        ),
         dedent(
             """\
             Failed to resolve all requirements for {target_description}:
@@ -403,7 +434,7 @@ def test_wheel_tag_mismatch(
 
 
 def test_requires_python_mismatch(
-    py37_target,  # type: Target
+    py39_target,  # type: Target
     py310_target,  # type: Target
 ):
     # type: (...) -> None
@@ -414,7 +445,7 @@ def test_requires_python_mismatch(
             LockedRequirement.create(
                 pin=pin("ansicolors", "1.1.7"),
                 artifact=artifact(
-                    url="https://example.org/ansicolors-1.1.7-py2.py3-none-any.whl",
+                    url="https://example.org/ansicolors-1.1.7-1buildtag-py2.py3-none-any.whl",
                     algorithm="blake256",
                     hash="cafebabe",
                 ),
@@ -440,11 +471,11 @@ def test_requires_python_mismatch(
     )
 
     assert_resolved(
-        locked_resolve.resolve(py37_target, [req("ansicolors==1.1.7")]),
+        locked_resolve.resolve(py39_target, [req("ansicolors==1.1.7")]),
         DownloadableArtifact.create(
             pin=pin("ansicolors", "1.1.7"),
             artifact=artifact(
-                url="https://example.org/ansicolors-1.1.7-py2.py3-none-any.whl",
+                url="https://example.org/ansicolors-1.1.7-1buildtag-py2.py3-none-any.whl",
                 algorithm="blake256",
                 hash="cafebabe",
             ),
@@ -453,6 +484,60 @@ def test_requires_python_mismatch(
     )
 
 
+def test_constraint_mismatch(
+    current_target,  # type: Target
+    ansicolors_simple,  # type: LockedResolve
+):
+    # type: (...) -> None
+    locked_resolve = LockedResolve(
+        platform_tag=Tag("cp37", "cp37m", "exotic"),
+        locked_requirements=locked_requirements(
+            LockedRequirement.create(
+                pin=pin("ansicolors", "1.1.7"),
+                artifact=artifact(
+                    url="https://example.org/ansicolors-1.1.7-1buildtag-py2.py3-none-any.whl",
+                    algorithm="blake256",
+                    hash="cafebabe",
+                ),
+            ),
+        ),
+    )
+
+    assert_error(
+        locked_resolve.resolve(
+            current_target, [req("ansicolors")], constraints=[req("ansicolors>=2")]
+        ),
+        dedent(
+            """\
+            Failed to resolve all requirements for {target_description}:
+
+            Configured with:
+                build: True
+                use_wheel: True
+
+            Dependency on ansicolors not satisfied, 1 incompatible candidate found:
+            1.) ansicolors 1.1.7 does not satisfy the following requirements:
+                >=2 (via: constraint)
+            """
+        ).format(target_description=current_target.render_description()),
+    )
+
+    assert_resolved(
+        locked_resolve.resolve(
+            current_target, [req("ansicolors")], constraints=[req("irrelevant==1.0.0")]
+        ),
+        DownloadableArtifact.create(
+            pin=pin("ansicolors", "1.1.7"),
+            artifact=artifact(
+                url="https://example.org/ansicolors-1.1.7-1buildtag-py2.py3-none-any.whl",
+                algorithm="blake256",
+                hash="cafebabe",
+            ),
+            satisfied_direct_requirements=requirements("ansicolors"),
+        ),
+    )
+
+
 def test_prefer_older_binary(current_target):
     # type: (Target) -> None
 
@@ -462,7 +547,7 @@ def test_prefer_older_binary(current_target):
             LockedRequirement.create(
                 pin=pin("ansicolors", "1.1.7"),
                 artifact=artifact(
-                    url="https://example.org/ansicolors-1.1.7-py2.py3-none-any.whl",
+                    url="https://example.org/ansicolors-1.1.7-1buildtag-py2.py3-none-any.whl",
                     algorithm="blake256",
                     hash="cafebabe",
                 ),
@@ -492,11 +577,15 @@ def test_prefer_older_binary(current_target):
     )
 
     assert_resolved(
-        locked_resolve.resolve(current_target, [req("ansicolors>1")], prefer_older_binary=True),
+        locked_resolve.resolve(
+            current_target,
+            [req("ansicolors>1")],
+            build_configuration=BuildConfiguration.create(prefer_older_binary=True),
+        ),
         DownloadableArtifact.create(
             pin=pin("ansicolors", "1.1.7"),
             artifact=artifact(
-                url="https://example.org/ansicolors-1.1.7-py2.py3-none-any.whl",
+                url="https://example.org/ansicolors-1.1.7-1buildtag-py2.py3-none-any.whl",
                 algorithm="blake256",
                 hash="cafebabe",
             ),
@@ -673,7 +762,11 @@ def test_multiple_errors(
     # type: (...) -> None
 
     assert_error(
-        cyclic_resolve.resolve(current_target, [req("A==1.0.1")], use_wheel=False),
+        cyclic_resolve.resolve(
+            current_target,
+            [req("A==1.0.1")],
+            build_configuration=BuildConfiguration.create(allow_wheels=False),
+        ),
         dedent(
             """\
             Failed to resolve all requirements for {target_description}:
@@ -692,3 +785,199 @@ def test_multiple_errors(
             """
         ).format(target_description=current_target.render_description()),
     )
+
+
+def test_resolved():
+    # type: () -> None
+
+    def assert_resolved(
+        expected_target_specificity,  # type: float
+        supported_tag_count,  # type: int
+        artifact_ranks,  # type: Iterable[int]
+    ):
+        # type: (...) -> None
+        direct_requirements = requirements("foo")
+
+        resolved_artifacts = tuple(
+            resolved_artifact(
+                project_name="foo",
+                version=str(rank),
+                artifact_basename="foo-{}.tar.gz".format(rank),
+                rank_value=rank,
+            )
+            for rank in artifact_ranks
+        )
+
+        downloadable_artifacts = tuple(
+            DownloadableArtifact.create(
+                pin=resolved_art.locked_requirement.pin,
+                artifact=resolved_art.artifact,
+                satisfied_direct_requirements=direct_requirements,
+            )
+            for resolved_art in resolved_artifacts
+        )
+
+        target = CompletePlatform.create(
+            MarkerEnvironment(),
+            CompatibilityTags.from_strings(
+                "py3-none-manylinux_2_{glibc_minor}_x86_64".format(glibc_minor=glibc_minor)
+                for glibc_minor in range(supported_tag_count)
+            ),
+        )
+
+        class DevNullFingerprinter(object):
+            def fingerprint(self, _artifacts):
+                # type: (Iterable[PartialArtifact]) -> Iterator[FileArtifact]
+                return iter(())
+
+        locked_resolve = LockedResolve.create(
+            resolved_requirements=(),
+            dist_metadatas=(),
+            fingerprinter=DevNullFingerprinter(),
+        )
+        assert Resolved[LockedResolve](
+            target_specificity=expected_target_specificity,
+            downloadable_artifacts=downloadable_artifacts,
+            source=locked_resolve,
+        ) == locked_resolve.create_resolved_artifacts(
+            target=target,
+            direct_requirements=direct_requirements,
+            resolved_artifacts=resolved_artifacts,
+        )
+
+    # For tag ranks of 1, 2, 1 should rank 100% target specific (best match) and 2 should rank 0%
+    # (worst match / universal)
+    assert_resolved(expected_target_specificity=1.0, supported_tag_count=2, artifact_ranks=[1])
+    assert_resolved(expected_target_specificity=0.0, supported_tag_count=2, artifact_ranks=[2])
+
+    # For tag ranks of 1, 2, 3, 2 lands in the middle and should be 50% target specific.
+    assert_resolved(expected_target_specificity=0.5, supported_tag_count=3, artifact_ranks=[2])
+
+
+def test_file_artifact():
+    # type: () -> None
+
+    artifact = Artifact.from_url(
+        url="file:///repo/A-1.0.0.tar.gz",
+        fingerprint=Fingerprint(algorithm="md5", hash="foo"),
+    )
+    assert isinstance(artifact, FileArtifact)
+    assert "A-1.0.0.tar.gz" == artifact.filename
+    assert artifact.is_source
+    assert frozenset() == frozenset(artifact.parse_tags())
+
+    artifact = Artifact.from_url(
+        url="https://example.org/ansicolors-1.1.7-py2.py3-none-any.whl",
+        fingerprint=Fingerprint(algorithm="sha1", hash="foo"),
+    )
+    assert isinstance(artifact, FileArtifact)
+    assert "ansicolors-1.1.7-py2.py3-none-any.whl" == artifact.filename
+    assert not artifact.is_source
+    assert frozenset((Tag("py2", "none", "any"), Tag("py3", "none", "any"))) == frozenset(
+        artifact.parse_tags()
+    )
+
+
+def test_vcs_artifact():
+    # type: () -> None
+
+    artifact = Artifact.from_url(
+        url="git+https://github.com/pex-tool/pex",
+        fingerprint=Fingerprint(algorithm="md5", hash="bar"),
+    )
+    assert isinstance(artifact, VCSArtifact)
+    assert VCS.Git is artifact.vcs
+    assert artifact.is_source
+    assert "pex @ git+https://github.com/pex-tool/pex" == artifact.as_unparsed_requirement(
+        ProjectName("pex")
+    )
+
+    artifact = Artifact.from_url(
+        url="hg+https://github.com/pex-tool/pex#egg=pex&subdirectory=.",
+        fingerprint=Fingerprint(algorithm="sha1", hash="bar"),
+    )
+    assert isinstance(artifact, VCSArtifact)
+    assert VCS.Mercurial is artifact.vcs
+    assert artifact.is_source
+    assert (
+        "hg+https://github.com/pex-tool/pex#egg=pex&subdirectory=."
+        == artifact.as_unparsed_requirement(ProjectName("pex"))
+    )
+
+
+def test_locked_requirement_mixed_artifacts_issue_2150():
+    # type: () -> None
+
+    file_artifact = FileArtifact(
+        url=ArtifactURL.parse("https://host/project.whl"),
+        fingerprint=Fingerprint(algorithm="md5", hash="foo"),
+        verified=False,
+        filename="a.whl",
+    )
+    vcs_artifact = VCSArtifact(
+        url=ArtifactURL.parse("git+https://host/a/project"),
+        fingerprint=Fingerprint(algorithm="sha1", hash="bar"),
+        vcs=VCS.Git,
+        verified=False,
+    )
+    local_project_artifact = LocalProjectArtifact(
+        url=ArtifactURL.parse("file:///tmp/project"),
+        fingerprint=Fingerprint(algorithm="sha256", hash="baz"),
+        verified=False,
+        directory="/tmp/project",
+    )
+    locked_requirement = LockedRequirement.create(
+        pin=Pin(ProjectName("project"), Version("0.1.0")),
+        artifact=local_project_artifact,
+        additional_artifacts=(file_artifact, vcs_artifact),
+    )
+    assert [local_project_artifact, vcs_artifact, file_artifact] == list(
+        locked_requirement.iter_artifacts()
+    ), (
+        "Expected the primary artifact to be 1st, and then the additional artifacts sorted by URL "
+        "(git+https:// comes before https://)."
+    )
+
+
+def test_locked_resolve_same_pins_mixed_primary_artifacts_issue_2150():
+    # type: () -> None
+
+    pin = Pin(ProjectName("project"), Version("0.1.0"))
+    file_artifact_requirement = LockedRequirement.create(
+        pin,
+        FileArtifact(
+            url=ArtifactURL.parse("https://host/project.whl"),
+            fingerprint=Fingerprint(algorithm="md5", hash="foo"),
+            verified=False,
+            filename="a.whl",
+        ),
+    )
+    vcs_artifact_requirement = LockedRequirement.create(
+        pin,
+        VCSArtifact(
+            url=ArtifactURL.parse("git+https://host/a/project"),
+            fingerprint=Fingerprint(algorithm="sha1", hash="bar"),
+            vcs=VCS.Git,
+            verified=False,
+        ),
+    )
+    local_project_artifact_requirement = LockedRequirement.create(
+        pin,
+        LocalProjectArtifact(
+            url=ArtifactURL.parse("file:///tmp/project"),
+            fingerprint=Fingerprint(algorithm="sha256", hash="baz"),
+            verified=False,
+            directory="/tmp/project",
+        ),
+    )
+    locked_requirements = [
+        file_artifact_requirement,
+        vcs_artifact_requirement,
+        local_project_artifact_requirement,
+    ]
+    locked_resolve = LockedResolve(locked_requirements=SortedTuple(locked_requirements))
+    assert [
+        local_project_artifact_requirement,
+        vcs_artifact_requirement,
+        file_artifact_requirement,
+    ] == list(locked_resolve.locked_requirements)
