@@ -1,28 +1,39 @@
-# Copyright 2014 Pants project contributors (see CONTRIBUTORS.md).
+# Copyright 2014 Pex project contributors.
 # Licensed under the Apache License, Version 2.0 (see LICENSE).
 
 import filecmp
 import os
+import re
 import stat
-import subprocess
 import sys
 import zipfile
+from contextlib import contextmanager
+from zipfile import ZipFile
 
 import pytest
 
-from pex.common import open_zip, safe_open, temporary_dir, touch
-from pex.compatibility import WINDOWS
+from pex.cache.dirs import CacheDir
+from pex.common import CopyMode, open_zip, safe_open, temporary_dir, touch
+from pex.compatibility import safe_commonpath
 from pex.executor import Executor
+from pex.fs import safe_rename
 from pex.layout import Layout
+from pex.os import WINDOWS
 from pex.pex import PEX
-from pex.pex_builder import CopyMode, PEXBuilder
-from pex.testing import PY_VER, built_wheel, make_bdist, make_env
-from pex.testing import write_simple_pex as write_pex
+from pex.pex_builder import Check, InvalidZipAppError, PEXBuilder
+from pex.pex_warnings import PEXWarning
 from pex.typing import TYPE_CHECKING
 from pex.variables import ENV
+from testing import IS_PYPY, NonDeterministicWalk, install_wheel, make_bdist, make_env, subprocess
+from testing import write_simple_pex as write_pex
+
+try:
+    from unittest import mock
+except ImportError:
+    import mock  # type: ignore[no-redef,import]
 
 if TYPE_CHECKING:
-    from typing import Any, Iterator, List, Set
+    from typing import Any, Iterator, List, Set, Text
 
 exe_main = """
 import sys
@@ -33,20 +44,9 @@ with open(sys.argv[1], 'w') as fp:
   fp.write('success')
 """
 
-wheeldeps_exe_main = """
-import sys
-from pyparsing import *
-from p1.my_module import do_something
-do_something()
-
-with open(sys.argv[1], 'w') as fp:
-  fp.write('success')
-"""
-
 
 def test_pex_builder():
     # type: () -> None
-    # test w/ and w/o zipfile dists
     with temporary_dir() as td, make_bdist("p1") as p1:
         pb = write_pex(td, exe_main, dists=[p1])
 
@@ -56,37 +56,6 @@ def test_pex_builder():
         with open(success_txt) as fp:
             assert fp.read() == "success"
 
-    # test w/ and w/o zipfile dists
-    with temporary_dir() as td1, temporary_dir() as td2, make_bdist("p1") as p1:
-        pb = write_pex(td1, exe_main, dists=[p1])
-
-        success_txt = os.path.join(td1, "success.txt")
-        PEX(td1, interpreter=pb.interpreter).run(args=[success_txt])
-        assert os.path.exists(success_txt)
-        with open(success_txt) as fp:
-            assert fp.read() == "success"
-
-
-@pytest.mark.skipif(
-    PY_VER >= (3, 10),
-    reason=(
-        "The pyparsing 2.1.10 distribution imports collections.MutableMapping which was (re)moved "
-        "in Python 3.10."
-    ),
-)
-def test_pex_builder_wheeldep():
-    # type: () -> None
-    """Repeat the pex_builder test, but this time include an import of something from a wheel that
-    doesn't come in importable form."""
-    with temporary_dir() as td, make_bdist("p1") as p1:
-        pyparsing_path = "./tests/example_packages/pyparsing-2.1.10-py2.py3-none-any.whl"
-        pb = write_pex(td, wheeldeps_exe_main, dists=[p1, pyparsing_path])
-        success_txt = os.path.join(td, "success.txt")
-        PEX(td, interpreter=pb.interpreter).run(args=[success_txt])
-        assert os.path.exists(success_txt)
-        with open(success_txt) as fp:
-            assert fp.read() == "success"
-
 
 def test_pex_builder_shebang():
     # type: () -> None
@@ -97,13 +66,12 @@ def test_pex_builder_shebang():
         return pb
 
     for pb in builder("foobar"), builder("#!foobar"):
-        for b in pb, pb.clone():
-            with temporary_dir() as td:
-                target = os.path.join(td, "foo.pex")
-                b.build(target)
-                expected_preamble = b"#!foobar\n"
-                with open(target, "rb") as fp:
-                    assert fp.read(len(expected_preamble)) == expected_preamble
+        with temporary_dir() as td:
+            target = os.path.join(td, "foo.pex")
+            pb.build(target)
+            expected_preamble = b"#!foobar\n"
+            with open(target, "rb") as fp:
+                assert fp.read(len(expected_preamble)) == expected_preamble
 
 
 def test_pex_builder_preamble():
@@ -176,25 +144,20 @@ def test_pex_builder_copy_or_link():
         def build_and_check(copy_mode):
             # type: (CopyMode.Value) -> None
             pb = PEXBuilder(copy_mode=copy_mode)
-            path = pb.path()
             pb.add_source(src, "exe.py")
 
-            path_clone = os.path.join(path, "__clone")
-            pb.clone(into=path_clone)
-
-            for root in path, path_clone:
-                s1 = os.stat(src)
-                s2 = os.stat(os.path.join(root, "exe.py"))
-                is_link = (s1[stat.ST_INO], s1[stat.ST_DEV]) == (s2[stat.ST_INO], s2[stat.ST_DEV])
-                if copy_mode == CopyMode.COPY:
-                    assert not is_link
-                else:
-                    # Since os.stat follows symlinks; so in CopyMode.SYMLINK, this just proves
-                    # the symlink points to the original file. Going further and checking path
-                    # and path_clone for the presence of a symlink (an os.islink test) is
-                    # trickier since a Linux hardlink of a symlink produces a symlink whereas a
-                    # macOS hardlink of a symlink produces a hardlink.
-                    assert is_link
+            s1 = os.stat(src)
+            s2 = os.stat(os.path.join(pb.path(), "exe.py"))
+            is_link = (s1[stat.ST_INO], s1[stat.ST_DEV]) == (s2[stat.ST_INO], s2[stat.ST_DEV])
+            if copy_mode == CopyMode.COPY:
+                assert not is_link
+            else:
+                # Since os.stat follows symlinks; so in CopyMode.SYMLINK, this just proves
+                # the symlink points to the original file. Going further and checking path
+                # and path_clone for the presence of a symlink (an os.islink test) is
+                # trickier since a Linux hardlink of a symlink produces a symlink whereas a
+                # macOS hardlink of a symlink produces a hardlink.
+                assert is_link
 
         build_and_check(CopyMode.LINK)
         build_and_check(CopyMode.COPY)
@@ -238,63 +201,26 @@ def test_pex_builder_deterministic_timestamp():
     pb = PEXBuilder()
     with temporary_dir() as td:
         target = os.path.join(td, "foo.pex")
-        pb.build(target, deterministic_timestamp=True)
+        pb.build(target, deterministic=True)
         with zipfile.ZipFile(target) as zf:
             assert all(zinfo.date_time == (1980, 1, 1, 0, 0, 0) for zinfo in zf.infolist())
 
 
-def test_pex_builder_from_requirements_pex():
-    # type: () -> None
-    def build_from_req_pex(path, req_pex):
-        # type: (str, str) -> PEXBuilder
-        pb = PEXBuilder(path=path)
-        pb.add_from_requirements_pex(req_pex)
-        with open(os.path.join(path, "exe.py"), "w") as fp:
-            fp.write(exe_main)
-        pb.set_executable(os.path.join(path, "exe.py"))
-        pb.freeze()
-        return pb
-
-    def verify(pb):
-        # type: (PEXBuilder) -> None
-        success_txt = os.path.join(pb.path(), "success.txt")
-        PEX(pb.path(), interpreter=pb.interpreter).run(args=[success_txt])
-        assert os.path.exists(success_txt)
-        with open(success_txt) as fp:
-            assert fp.read() == "success"
-
-    # Build from pex dir.
-    with temporary_dir() as td2:
-        with temporary_dir() as td1, make_bdist("p1") as p1:
-            pb1 = write_pex(td1, dists=[p1])
-            pb2 = build_from_req_pex(td2, pb1.path())
-        verify(pb2)
-
-    # Build from .pex file.
-    with temporary_dir() as td4:
-        with temporary_dir() as td3, make_bdist("p1") as p1:
-            pb3 = write_pex(td3, dists=[p1])
-            target = os.path.join(td3, "foo.pex")
-            pb3.build(target)
-            pb4 = build_from_req_pex(td4, target)
-        verify(pb4)
-
-
 def test_pex_builder_script_from_pex_path(tmpdir):
     # type: (Any) -> None
 
     pex_with_script = os.path.join(str(tmpdir), "script.pex")
-    with built_wheel(
+    with make_bdist(
         name="my_project",
         entry_points={"console_scripts": ["my_app = my_project.my_module:do_something"]},
-    ) as my_whl:
+    ) as dist:
         pb = PEXBuilder()
-        pb.add_dist_location(my_whl)
+        pb.add_dist_location(dist.location)
         pb.build(pex_with_script)
 
     pex_file = os.path.join(str(tmpdir), "app.pex")
     pb = PEXBuilder()
-    pb.info.pex_path = pex_with_script
+    pb.info.pex_path = [pex_with_script]
     pb.set_script("my_app")
     pb.build(pex_file)
 
@@ -305,11 +231,11 @@ def test_pex_builder_setuptools_script(tmpdir):
     # type: (Any) -> None
 
     pex_file = os.path.join(str(tmpdir), "app.pex")
-    with built_wheel(
+    with make_bdist(
         name="my_project",
-    ) as my_whl:
+    ) as dist:
         pb = PEXBuilder()
-        pb.add_dist_location(my_whl)
+        pb.add_dist_location(dist.location)
         pb.set_script("shell_script")
         pb.build(pex_file)
 
@@ -326,10 +252,10 @@ def test_pex_builder_packed(tmpdir):
     source_file = os.path.join(str(tmpdir), "src")
     touch(source_file)
 
-    with ENV.patch(PEX_ROOT=pex_root), built_wheel(name="my_project") as my_whl:
+    with ENV.patch(PEX_ROOT=pex_root), make_bdist(name="my_project") as dist:
         pb = PEXBuilder(copy_mode=CopyMode.SYMLINK)
         pb.add_source(source_file, "a.file")
-        pb.add_dist_location(my_whl)
+        pb.add_dist_location(dist.location)
         pb.set_script("shell_script")
         pb.build(pex_app, layout=Layout.PACKED)
 
@@ -340,8 +266,8 @@ def test_pex_builder_packed(tmpdir):
     spread_dist_bootstrap = os.path.join(pex_app, pb.info.bootstrap)
     assert zipfile.is_zipfile(spread_dist_bootstrap)
 
-    cached_bootstrap_zip = os.path.join(
-        pex_root, "bootstrap_zips", pb.info.bootstrap_hash, pb.info.bootstrap
+    cached_bootstrap_zip = CacheDir.BOOTSTRAP_ZIPS.path(
+        pb.info.bootstrap_hash, pb.info.bootstrap, pex_root=pex_root
     )
     assert zipfile.is_zipfile(cached_bootstrap_zip)
 
@@ -351,7 +277,7 @@ def test_pex_builder_packed(tmpdir):
     for root, dirs, files in os.walk(pex_app, followlinks=False):
         for f in files:
             path = os.path.join(root, f)
-            assert not os.path.islink(path) or pex_app == os.path.commonprefix(
+            assert not os.path.islink(path) or pex_app == safe_commonpath(
                 [pex_app, os.path.realpath(path)]
             ), (
                 "All packed layout files should be real files inside the packed layout root that "
@@ -364,7 +290,7 @@ def test_pex_builder_packed(tmpdir):
     spread_dist_zip = os.path.join(pex_app, pb.info.internal_cache, location)
     assert zipfile.is_zipfile(spread_dist_zip)
 
-    cached_dist_zip = os.path.join(pex_root, "installed_wheel_zips", sha, location)
+    cached_dist_zip = CacheDir.PACKED_WHEELS.path(sha, location, pex_root=pex_root)
     assert zipfile.is_zipfile(cached_dist_zip)
 
     assert filecmp.cmp(spread_dist_zip, cached_dist_zip, shallow=False)
@@ -388,25 +314,34 @@ def test_pex_builder_exclude_bootstrap_testing(
     pb.build(pex_path, layout=layout)
 
     bootstrap_location = os.path.join(pex_path, pb.info.bootstrap)
-    bootstrap_files = set()  # type: Set[str]
+    bootstrap_files = set()  # type: Set[Text]
+    vendor_dir_relpath = os.path.join("pex", "vendor", "_vendored")
     if Layout.ZIPAPP == layout:
         with open_zip(pex_path) as zf:
+            vendor_dir = os.path.join(pb.info.bootstrap, vendor_dir_relpath)
             bootstrap_files.update(
                 os.path.relpath(f, pb.info.bootstrap)
                 for f in zf.namelist()
-                if f.startswith(pb.info.bootstrap)
+                if f.startswith(pb.info.bootstrap) and not f.startswith(vendor_dir)
             )
     elif Layout.PACKED == layout:
         with open_zip(bootstrap_location) as zf:
-            bootstrap_files.update(zf.namelist())
+            bootstrap_files.update(
+                name for name in zf.namelist() if not name.startswith(vendor_dir_relpath)
+            )
     else:
-        bootstrap_files.update(
-            os.path.relpath(os.path.join(root, f), bootstrap_location)
-            for root, _, files in os.walk(bootstrap_location)
-            for f in files
-        )
+        for root, dirs, files in os.walk(bootstrap_location):
+            dirs[:] = [
+                d
+                for d in dirs
+                if os.path.relpath(os.path.join(root, d), bootstrap_location) != vendor_dir_relpath
+            ]
+            for f in files:
+                bootstrap_files.add(os.path.relpath(os.path.join(root, f), bootstrap_location))
 
-    assert {"pex/pex_bootstrapper.py", "pex/pex_info.py", "pex/pex.py"}.issubset(
+    assert {
+        os.path.join("pex", f) for f in ("pex_bootstrapper.py", "pex_info.py", "pex.py")
+    }.issubset(
         bootstrap_files
     ), "Expected the `.bootstrap` to contain at least some of the key Pex runtime modules."
     assert not [
@@ -414,6 +349,23 @@ def test_pex_builder_exclude_bootstrap_testing(
     ], "Expected testing support files to be stripped from the Pex `.bootstrap`."
 
 
+def test_pex_builder_deterministic_pex_info(
+    tmpdir,  # type: Any
+):
+    # type: (...) -> None
+
+    pex_path = os.path.join(str(tmpdir), "empty.pex")
+
+    with mock.patch("os.walk", new=NonDeterministicWalk()):
+        pb_one = PEXBuilder()
+        pb_one.build(pex_path)
+
+        pb_two = PEXBuilder()
+        pb_two.build(pex_path)
+
+        assert pb_one.info.as_json_dict() == pb_two.info.as_json_dict()
+
+
 @pytest.mark.parametrize(
     "strip_pex_env", [pytest.param(True, id="StripPexEnv"), pytest.param(False, id="NoStripPexEnv")]
 )
@@ -495,3 +447,149 @@ def test_pex_env_var_issues_1485(
     assert_pex_env_var_nested()
     assert_pex_env_var_nested(PEX_VENV=False)
     assert_pex_env_var_nested(PEX_VENV=True)
+
+
+@pytest.mark.parametrize(
+    "layout", [pytest.param(layout, id=layout.value) for layout in (Layout.PACKED, Layout.ZIPAPP)]
+)
+def test_build_compression(
+    tmpdir,  # type: Any
+    layout,  # type: Layout.Value
+    pex_wheel,  # type: str
+):
+    # type: (...) -> None
+
+    pex_root = os.path.join(str(tmpdir), "pex_root")
+
+    pb = PEXBuilder()
+    pb.info.pex_root = pex_root
+    with ENV.patch(PEX_ROOT=pex_root):
+        pb.add_dist_location(install_wheel(pex_wheel).location)
+    exe = os.path.join(str(tmpdir), "exe.py")
+    with open(exe, "w") as fp:
+        fp.write("import pex; print(pex.__file__)")
+    pb.set_executable(exe)
+
+    def assert_pex(pex):
+        # type: (str) -> None
+        output = subprocess.check_output(
+            args=[sys.executable, pex], env=make_env(PEX_ROOT=None)
+        ).decode("utf-8")
+        assert output.startswith(pex_root), pex_root + " vs:\n" + output
+
+    compressed_pex = os.path.join(str(tmpdir), "compressed.pex")
+    pb.build(compressed_pex, layout=layout)
+    assert_pex(compressed_pex)
+
+    uncompressed_pex = os.path.join(str(tmpdir), "uncompressed.pex")
+    pb.build(uncompressed_pex, layout=layout, compress=False)
+    assert_pex(uncompressed_pex)
+
+    def size(pex):
+        # type: (str) -> int
+        if os.path.isfile(pex):
+            return os.path.getsize(pex)
+        return sum(
+            os.path.getsize(os.path.join(root, f)) for root, _, files in os.walk(pex) for f in files
+        )
+
+    assert size(compressed_pex) < size(uncompressed_pex)
+
+
+@pytest.mark.skipif(
+    sys.version_info == (3, 13, 0, "alpha", 6),
+    reason=(
+        "The 3.13.0a6 release has a bug in its zipimporter ZIP64 handling, see: "
+        "https://github.com/python/cpython/issues/118107"
+    ),
+)
+@pytest.mark.skipif(
+    subprocess.call(args=["cat", os.devnull]) != 0,
+    reason="The cat binary is required for this test.",
+)
+def test_check(tmpdir):
+    # type: (Any) -> None
+
+    def assert_perform_check_zip64_handling(
+        zipapp,  # type: str
+        test_run=True,  # type: bool
+    ):
+        # type: (...) -> None
+        assert Check.NONE.perform_check(Layout.ZIPAPP, zipapp) is None
+        assert Check.ERROR.perform_check(Layout.PACKED, zipapp) is None
+        assert Check.ERROR.perform_check(Layout.LOOSE, zipapp) is None
+
+        # Python 3.13 newly supports ZIP64 in `zipimport.zipimporter`.
+        if sys.version_info >= (3, 13):
+            assert Check.WARN.perform_check(Layout.ZIPAPP, zipapp) is True
+            assert Check.ERROR.perform_check(Layout.ZIPAPP, zipapp) is True
+            if test_run:
+                subprocess.check_call(args=[sys.executable, zipapp])
+        else:
+            expected_error_message_re = r"The PEX zip at {path} is not a valid zipapp: ".format(
+                path=re.escape(zipapp)
+            )
+            with pytest.warns(PEXWarning, match=expected_error_message_re):
+                assert Check.WARN.perform_check(Layout.ZIPAPP, zipapp) is False
+            with pytest.raises(InvalidZipAppError, match=expected_error_message_re):
+                Check.ERROR.perform_check(Layout.ZIPAPP, zipapp)
+
+            if test_run:
+                assert subprocess.call(args=[sys.executable, zipapp]) != 0
+
+    @contextmanager
+    def write_zipapp(path):
+        # type: (str) -> Iterator[ZipFile]
+        with open_zip(path, "w") as zip_file:
+            yield zip_file
+            zip_file.writestr("__main__.py", "print('BOOTED')")
+
+    # N.B.: This test creates very large files in /tmp; so to relieve possible memory pressure in
+    # tmpfs-based /tmp filesystems, we unlink files as soon as we don't need them instead of waiting
+    # for tmpdir fixture cleanup at the end of the test.
+
+    file_too_big = os.path.join(str(tmpdir), "file_too_big.py")
+    with open(file_too_big, "w") as fp:
+        fp.write("\n")
+    accum = file_too_big + ".accum"
+    for _ in range(32):
+        safe_rename(file_too_big, accum)
+        with open(file_too_big, "wb") as dest:
+            subprocess.check_call(args=["cat", accum, accum], stdout=dest.fileno())
+    os.unlink(accum)
+    assert os.path.getsize(file_too_big) > 0xFFFFFFFF, (
+        "ZIP64 must be used if file sizes are bigger than 0xFFFFFFFF per "
+        "https://pkware.cachefly.net/webdocs/casestudies/APPNOTE.TXT 4.3.9.2"
+    )
+
+    zipapp_too_big = os.path.join(str(tmpdir), "too-big.pyz")
+    with write_zipapp(zipapp_too_big) as zf:
+        zf.write(file_too_big, os.path.basename(file_too_big))
+    os.unlink(file_too_big)
+    assert_perform_check_zip64_handling(
+        zipapp_too_big,
+        # N.B.: PyPy < 3.8 hangs when trying to run a zip64 app with a too-large file entry.
+        test_run=not IS_PYPY or sys.version_info[:2] >= (3, 8),
+    )
+    os.unlink(zipapp_too_big)
+
+    zipapp_too_many_entries = os.path.join(str(tmpdir), "too-many-entries.pyz")
+    with write_zipapp(zipapp_too_many_entries) as zf:
+        # N.B.: ZIP64 must be used if the number of central directory records is greater than
+        # 0xFFFF per https://pkware.cachefly.net/webdocs/casestudies/APPNOTE.TXT 4.4.21
+        for x in range(0xFFFF):
+            zf.writestr(
+                "module{x}.py".format(x=x),
+                "# This is python module number {x}.".format(x=x).encode("utf-8"),
+            )
+    assert_perform_check_zip64_handling(zipapp_too_many_entries)
+
+    zipapp_ok = os.path.join(str(tmpdir), "ok.pyz")
+    with write_zipapp(zipapp_ok):
+        pass
+    for layout in Layout.values():
+        for check in Check.values():
+            assert check.perform_check(layout, zipapp_ok) is (
+                True if layout is Layout.ZIPAPP and check is not Check.NONE else None
+            )
+    assert b"BOOTED\n" == subprocess.check_output(args=[sys.executable, zipapp_ok])
